import os
import os.path
import secrets
import string
import sys

import django.dispatch
from django.apps import apps
from django.conf import settings
from django.contrib.auth.models import Group
from django.core.exceptions import FieldDoesNotExist
from django.core.management import call_command
from django.core.management.commands.runserver import Command as BaseCommand


try:
    import devpy.develop as logger
except ImportError:
    import logging
    logger = logging.getLogger('dev')

signal = django.dispatch.Signal()


def rnpw(num=28):
    return ''.join(secrets.choice(
        string.ascii_uppercase + string.digits) for _ in range(num))


class Command(BaseCommand):
    help = 'Start development environment'

    def handle(self, *args, **options):
        call_command('migrate')
        self.createusers()
        self.runserver(*args, **options)

    def runserver(self, *args, **options):
        try:
            pid = os.fork()
        except OSError:
            sys.exit(1)

        if pid == 0:
            signal.send(sender=self)
        else:
            options['nothreading'] = True
            super().handle(*args, **options)

    def createusers(self):
        user_model = apps.get_model(settings.AUTH_USER_MODEL)

        def createuser(username, *groups, **defaults):
            for key in list(defaults.keys()):
                try:
                    user_model._meta.get_field(key)
                except FieldDoesNotExist:
                    defaults.pop(key)

            user, created = user_model.objects.update_or_create(
                username=username,
                defaults=defaults,
            )

            if created:
                user.set_password(username)
                user.save()
            logger.warning('\n{}\nLogin with {} / {}\n'.format(
                '*' * 12, username, username))

            return user, created

        createuser('dev', is_staff=True, is_superuser=True)
        createuser('staff', is_staff=True)
        createuser('user')

        for group in Group.objects.all():
            createuser(
                group.name.lower().replace(' ', ''),
                group,
            )



import copy
import urllib

from django import template


def pagination_filter_params(data):
    data = copy.deepcopy(data)
    if 'page' in data:
        data.pop('page')
    return urllib.parse.urlencode(data)


def render_form(form):
    tpl = ['{% load material_form %}{% form form=form ']
    context = dict(form=form)

    if getattr(form, '_layout', None):
        tpl.append('layout=layout')
        context['layout'] = form._layout

    tpl.append('%}{% endform %}')

    return template.Template(
        ' '.join(tpl)
    ).render(template.Context(context))



from crudlfap import shortcuts as crudlfap

from django.conf import settings
from django.urls import include, re_path
from django.views.static import serve

urlpatterns = [
    crudlfap.site.urlpattern,
    re_path(
        r'^%s(?P<path>.*)$' % settings.STATIC_URL.lstrip('/'),
        serve,
        kwargs=dict(document_root=settings.STATIC_ROOT)
    ),
]
if 'debug_toolbar' in settings.INSTALLED_APPS and settings.DEBUG:
    import debug_toolbar
    urlpatterns += [
        re_path(r'^__debug__/', include(debug_toolbar.urls)),
    ]



import re

from crudlfap_example.artist.models import Artist

from django.views import generic

from .route import Route
from .router import Router


def test_model():
    route = Route.clone(model=Artist)
    assert route.model == Artist


def test_model_router_fallback():
    route = Router(Artist, fields='__all__', views=[Route]).views[0]
    assert route.model == Artist


def test_urlname():
    route = Route.clone(urlname='foo')
    assert route.urlname == 'foo'


def test_urlname_from_name():
    class FooRoute(Route):
        pass
    assert FooRoute.urlname == 'foo'


def test_urlpath():
    route = Route.clone(urlpath='foo')
    assert route.urlpath == 'foo'


def test_urlpath_from_urlname():
    class FooRoute(Route):
        pass
    assert FooRoute.urlpath == 'foo'


def test_urlpattern():
    class DetailView(generic.DetailView, Route):
        model = Artist
    p = DetailView.urlpattern
    assert p.name == 'detail'
    assert p.pattern.regex == re.compile('^detail$')
    assert p.callback.view_class == DetailView
